import { nanoid } from 'nanoid';
import cloneDeep from 'lodash.clonedeep';

const arrayRandom = (range) => {
  let start = range[0];
  let end = range[1];
  const arr = [];
  let i = start;
  while (i < end) {
    arr.push(Math.floor(i+1));
    i++;
  }
  const len = arr.length - 1;
  const idx = Math.floor(Math.random() * len);
  return arr[idx];
};

function generate(item) {
  if (typeof item === "string") {
    return `${item}-${nanoid()}`;
  }
  if (typeof item === "number") {
    const num = 5;
    const range = [Math.floor(item - num), Math.floor(item + num)];
    return arrayRandom(range)
  }
  if (typeof item === "boolean") {
    return Math.random() > 0.5;
  }
  if (
    Array.isArray(item) ||
    Object.prototype.toString.call(item) === "[object Object]"
  ) {
    return muskClone(item);
  }
  return cloneDeep(item);
}

function muskClone(source) {
  // null
  if (!source) {
    return;
  }
  // limit to Array and Object
  if (
    !Array.isArray(source) &&
    Object.prototype.toString.call(source) !== "[object Object]"
  ) {
    return;
  }
  let target = null;
  // Array
  if (Array.isArray(source)) {
    target = [];
    for (const value of source) {
      target.push(generate(value));
    }
  }
  // Object
  if (Object.prototype.toString.call(source) === "[object Object]") {
    target = {};
    for (const [key, value] of Object.entries(source)) {
      target[key] = generate(value);
    }
  }

  return target;
}

export default muskClone;
